aboutsummaryrefslogtreecommitdiffstats
path: root/LibOVR/Src/Kernel/OVR_System.cpp
diff options
context:
space:
mode:
Diffstat (limited to 'LibOVR/Src/Kernel/OVR_System.cpp')
-rw-r--r--LibOVR/Src/Kernel/OVR_System.cpp86
1 files changed, 77 insertions, 9 deletions
diff --git a/LibOVR/Src/Kernel/OVR_System.cpp b/LibOVR/Src/Kernel/OVR_System.cpp
index 3144ade..66c764a 100644
--- a/LibOVR/Src/Kernel/OVR_System.cpp
+++ b/LibOVR/Src/Kernel/OVR_System.cpp
@@ -6,16 +6,16 @@ Content : General kernel initialization/cleanup, including that
Created : September 19, 2012
Notes :
-Copyright : Copyright 2014 Oculus VR, Inc. All Rights reserved.
+Copyright : Copyright 2014 Oculus VR, LLC All Rights reserved.
-Licensed under the Oculus VR Rift SDK License Version 3.1 (the "License");
+Licensed under the Oculus VR Rift SDK License Version 3.2 (the "License");
you may not use the Oculus VR Rift SDK except in compliance with the License,
which is provided at the time of installation or download, or which
otherwise accompanies this software in either electronic or hard copy form.
You may obtain a copy of the License at
-http://www.oculusvr.com/licenses/LICENSE-3.1
+http://www.oculusvr.com/licenses/LICENSE-3.2
Unless required by applicable law or agreed to in writing, the Oculus VR SDK
distributed under the License is distributed on an "AS IS" BASIS,
@@ -28,10 +28,56 @@ limitations under the License.
#include "OVR_System.h"
#include "OVR_Threads.h"
#include "OVR_Timer.h"
+#include "../Displays/OVR_Display.h"
+#ifdef OVR_OS_WIN32
+#include "../Displays/OVR_Win32_ShimFunctions.h"
+#endif
namespace OVR {
-// ***** OVR::System Implementation
+#ifdef OVR_OS_WIN32
+extern bool anyRiftsInExtendedMode();
+#endif
+
+// Stack of destroy listeners (push/pop semantics)
+static SystemSingletonInternal *SystemShutdownListenerStack = 0;
+static Lock stackLock;
+static bool DisplayShimInitialized = false;
+
+void SystemSingletonInternal::PushDestroyCallbacks()
+{
+ Lock::Locker locker(&stackLock);
+
+ // Push listener onto the stack
+ NextSingleton = SystemShutdownListenerStack;
+ SystemShutdownListenerStack = this;
+}
+
+void System::DirectDisplayInitialize()
+{
+#ifdef OVR_OS_WIN32
+ // Set up display code for Windows
+ Win32::DisplayShim::GetInstance();
+
+ // This code will look for the first display. If it's a display
+ // that's extending the destkop, the code will assume we're in
+ // compatibility mode. Compatibility mode prevents shim loading
+ // and renders only to extended Rifts.
+ // If we find a display and it's application exclusive,
+ // we load the shim so we can render to it.
+ // If no display is available, we revert to whatever the
+ // driver tells us we're in
+
+ bool anyExtendedRifts = anyRiftsInExtendedMode() || Display::InCompatibilityMode( false );
+
+ DisplayShimInitialized = Win32::DisplayShim::GetInstance().Initialize(anyExtendedRifts);
+#endif
+}
+
+bool System::DirectDisplayEnabled()
+{
+ return DisplayShimInitialized;
+}
// Initializes System core, installing allocator.
void System::Init(Log* log, Allocator *palloc)
@@ -41,6 +87,8 @@ void System::Init(Log* log, Allocator *palloc)
Log::SetGlobalLog(log);
Timer::initializeTimerSystem();
Allocator::setInstance(palloc);
+ Display::Initialize();
+ DirectDisplayInitialize();
}
else
{
@@ -52,13 +100,33 @@ void System::Destroy()
{
if (Allocator::GetInstance())
{
- // Wait for all threads to finish; this must be done so that memory
- // allocator and all destructors finalize correctly.
+#ifdef OVR_OS_WIN32
+ Win32::DisplayShim::GetInstance().Shutdown();
+#endif
+
+ // Invoke all of the post-finish callbacks (normal case)
+ for (SystemSingletonInternal *listener = SystemShutdownListenerStack; listener; listener = listener->NextSingleton)
+ {
+ listener->OnThreadDestroy();
+ }
+
#ifdef OVR_ENABLE_THREADS
- Thread::FinishAllThreads();
+ // Wait for all threads to finish; this must be done so that memory
+ // allocator and all destructors finalize correctly.
+ Thread::FinishAllThreads();
#endif
- // Shutdown heap and destroy SysAlloc singleton, if any.
+ // Invoke all of the post-finish callbacks (normal case)
+ for (SystemSingletonInternal *next, *listener = SystemShutdownListenerStack; listener; listener = next)
+ {
+ next = listener->NextSingleton;
+
+ listener->OnSystemDestroy();
+ }
+
+ SystemShutdownListenerStack = 0;
+
+ // Shutdown heap and destroy SysAlloc singleton, if any.
Allocator::GetInstance()->onSystemShutdown();
Allocator::setInstance(0);
@@ -77,5 +145,5 @@ bool System::IsInitialized()
return Allocator::GetInstance() != 0;
}
-} // OVR
+} // namespace OVR